<?php

namespace Teknisa\Libs\Controller;

use Teknisa\Libs\Exception\CustomException;
use Teknisa\Libs\Util\Utilities;
use Teknisa\Libs\Util\InstanceProvider;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DataSource\DataSet;
use Teknisa\Libs\Service\Login as LoginService;
use Teknisa\Libs\Service\Api as ApiService;
use Teknisa\Libs\Util\Utilities as LibUtilities;

class Login {
    /** @var LoginService $loginService */
    private $loginService;
    /** @var ApiService $apiService */
    private $apiService;

    /**
     * @param LoginService $loginService
     * @param ApiService $apiService
     */
    public function __construct(LoginService $loginService, ApiService $apiService) {
        $this->loginService = $loginService;
        $this->apiService   = $apiService;
    }

    /**
     * @param Request\Filter $request
     * @param Response $response
     * @throws \Exception
     */
    public function startSession(Request\Filter $request, Response $response) {
        try {
            $conditions = $request->getFilterCriteria()->getConditions();
            $isLogged   = Utilities::getValueInFilterDataSourceByName($conditions, 'IS_LOGGED');
            $loginUser  = Utilities::getValueInFilterDataSourceByName($conditions, 'USER');
            $loginToken = Utilities::getValueInFilterDataSourceByName($conditions, 'LOGIN_TOKEN');
            $loginLang  = Utilities::getValueInFilterDataSourceByName($conditions, 'LOGIN_LANGUAGE');

            $modulesVersionError = null;
            try {
                $this->apiService->getModulesVersion();
            } catch(CustomException $e) {
                $modulesVersionError = array(
                    'MESSAGE_CODE'   => $e->getMessage(),
                    'MESSAGE_PARAMS' => $e->getParams()
                );
            }

            $loginUserInfo = @json_decode(LibUtilities::zeedhiDecrypt($loginUser), true);
            if(empty($loginUserInfo)) {
                $loginUserInfo = InstanceProvider::getSession()->get('LOGIN_USER_INFO');
            } else {
                unset($loginUserInfo['CURRENT_SECONDS']);
            }
            if(isset($loginUserInfo['USER_DATA']) && !empty($loginUserInfo['USER_DATA'])) {
                $userData = $loginUserInfo['USER_DATA'];
            } else {
                $userData = $this->apiService->getUserData($loginUserInfo['USER_EMAIL'], null);
            }
            $userData = $this->loginService->restoreSession($userData, $loginUserInfo, $isLogged, $modulesVersionError, $loginToken, $loginLang);
            $response->addDataSet(new DataSet('system', $userData));
        } catch(CustomException $e) {
            $response->setError(Utilities::getErrorObjectFromException($e));
        }
    }

    /**
     * @param Request\Filter $request
     * @param Response $response
     * @throws \Exception
     */
    public function logout(Request\Filter $request, Response $response) {
        try {
            $conditions = $request->getFilterCriteria()->getConditions();
            $project    = Utilities::getValueInFilterDataSourceByName($conditions, 'PROJECT');
            $eraseHash  = Utilities::getValueInFilterDataSourceByName($conditions, 'ERASE_HASH');
            $hash       = Utilities::getValueInFilterDataSourceByName($conditions, 'HASH');
            $this->loginService->logout($project, $eraseHash, $hash);
        } catch(\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), Response\Message::TYPE_ERROR));
        }
    }

    /**
     * @param Request\Filter $request
     * @param Response $response
     * @throws \Exception
     */
    public function validateExpiredSession(Request\Filter $request, Response $response) {
        try {
            $conditions  = $request->getFilterCriteria()->getConditions();
            $timeout     = Utilities::getValueInFilterDataSourceByName($conditions, 'TIMEOUT');
            $sessionInfo = $this->loginService->validateExpiredSession($timeout);
            $response->addDataSet(new DataSet('expiredSessionInfo', $sessionInfo));
        } catch(\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), Response\Message::TYPE_ERROR));
        }
    }

    /**
     * @param Request\Filter $request
     * @param Response $response
     * @throws \Exception
     */
    public function updateSession(Request\Filter $request, Response $response) {
        try {
            $conditions         = $request->getFilterCriteria()->getConditions();
            $user               = Utilities::getValueInFilterDataSourceByName($conditions, 'USER');
            $accessProfile      = Utilities::getValueInFilterDataSourceByName($conditions, 'ACCESS_PROFILE');
            $otherSessionValues = Utilities::getValueInFilterDataSourceByName($conditions, 'OTHER_SESSION_VALUES');
            $loginUser          = Utilities::getValueInFilterDataSourceByName($conditions, 'LOGIN_USER');
            $userSession        = $this->loginService->updateSession($user, $accessProfile, $otherSessionValues, $loginUser);
            $response->addDataSet(new DataSet('sessionUpdated', $userSession));
        } catch(\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), Response\Message::TYPE_ERROR));
        }
    }
}